@@ -88,19 +88,12 @@ class ScaleIOConnector(base.BaseLinuxConnector):
             raise exception.BrickException(message=msg)
 
     @staticmethod
-    def _get_password_token(connection_properties):
-        # In old connection format we had the password and token in properties
-        if 'serverPassword' in connection_properties:
-            return (connection_properties['serverPassword'],
-                    connection_properties['serverToken'])
-
-        # The new format reads password from file and doesn't have the token
+    def _get_connector_password(config_group, failed_over):
         LOG.info("Get ScaleIO connector password from configuration file")
         try:
-            password = priv_scaleio.get_connector_password(
-                CONNECTOR_CONF_PATH,
-                connection_properties['config_group'])
-            return password, None
+            return priv_scaleio.get_connector_password(CONNECTOR_CONF_PATH,
+                                                       config_group,
+                                                       failed_over)
         except Exception as e:
             msg = _("Error getting ScaleIO connector password from "
                     "configuration file: %s") % e
@@ -327,8 +320,10 @@ class ScaleIOConnector(base.BaseLinuxConnector):
         self.server_ip = connection_properties['serverIP']
         self.server_port = connection_properties['serverPort']
         self.server_username = connection_properties['serverUsername']
-        self.server_password, self.server_token = self._get_password_token(
-            connection_properties)
+        self.server_password = self._get_connector_password(
+            connection_properties['config_group'],
+            connection_properties['failed_over'],
+        )
         self.iops_limit = connection_properties['iopsLimit']
         self.bandwidth_limit = connection_properties['bandwidthLimit']
         device_info = {'type': 'block',
