@@ -101,38 +101,34 @@ class LinuxSCSI(executor.Executor):
         raise exception.VolumePathNotRemoved(volume_path=exist)
 
     def get_device_info(self, device):
-        (out, _err) = self._execute('sg_scan', device, run_as_root=True,
-                                    root_helper=self._root_helper)
         dev_info = {'device': device, 'host': None,
                     'channel': None, 'id': None, 'lun': None}
+        # The input argument 'device' can be of 2 types:
+        # (a) /dev/disk/by-path/XXX which is a symlink to /dev/sdX device
+        # (b) /dev/sdX
+        # If it's a symlink, get the /dev/sdX name first
+        if os.path.islink(device):
+            device = '/dev/' + os.readlink(device).split('/')[-1]
+        # Else it's already a /dev/sdX device.
+        # Then get it from lsscsi output
+        (out, _err) = self._execute('lsscsi')
         if out:
-            line = out.strip()
-            line = line.replace(device + ": ", "")
-            info = line.split(" ")
-
-            for item in info:
-                if '=' in item:
-                    pair = item.split('=')
-                    dev_info[pair[0]] = pair[1]
-                elif 'scsi' in item:
-                    dev_info['host'] = item.replace('scsi', '')
+            for line in out.strip().split('\n'):
+                # The last column of lsscsi is device name
+                if line.split()[-1] == device:
+                    # The first column of lsscsi is [H:C:T:L]
+                    hctl_info = line.split()[0].strip('[]').split(':')
+                    dev_info['host'] = hctl_info[0]
+                    dev_info['channel'] = hctl_info[1]
+                    dev_info['id'] = hctl_info[2]
+                    dev_info['lun'] = hctl_info[3]
+                    break
 
+        LOG.debug('dev_info=%s', str(dev_info))
         return dev_info
 
-    def get_sysfs_wwn(self, device_names, mpath=None):
+    def get_sysfs_wwn(self, device_names):
         """Return the wwid from sysfs in any of devices in udev format."""
-        # If we have a multipath DM we know that it has found the WWN
-        if mpath:
-            # We have the WWN in /uuid even with friendly names, unline /name
-            try:
-                with open('/sys/block/%s/dm/uuid' % mpath) as f:
-                    # Contents are matph-WWN, so get the part we want
-                    wwid = f.read().strip()[6:]
-                    if wwid:  # Check should not be needed, but just in case
-                        return wwid
-            except Exception as exc:
-                LOG.warning('Failed to read the DM uuid: %s', exc)
-
         wwid = self.get_sysfs_wwid(device_names)
         glob_str = '/dev/disk/by-id/scsi-'
         wwn_paths = glob.glob(glob_str + '*')
@@ -140,34 +136,16 @@ class LinuxSCSI(executor.Executor):
         if wwid and glob_str + wwid in wwn_paths:
             return wwid
 
-        # If we have multiple designators use symlinks to find out the wwn
-        device_names = set(device_names)
+        # If we have multiple designators follow the symlinks
         for wwn_path in wwn_paths:
             try:
                 if os.path.islink(wwn_path) and os.stat(wwn_path):
                     path = os.path.realpath(wwn_path)
-                    if path.startswith('/dev/'):
-                        name = path[5:]
-                        # Symlink may point to the multipath dm if the attach
-                        # was too fast or we took long to check it. Check
-                        # devices belonging to the multipath DM.
-                        if name.startswith('dm-'):
-                            # Get the devices that belong to the DM
-                            slaves_path = '/sys/class/block/%s/slaves' % name
-                            dm_devs = os.listdir(slaves_path)
-                            # This is the right wwn_path if the devices we have
-                            # attached belong to the dm we followed
-                            if device_names.intersection(dm_devs):
-                                break
-
-                        # This is the right wwn_path if  devices we have
-                        elif name in device_names:
-                            break
+                    if path.startswith('/dev/') and path[5:] in device_names:
+                        return wwn_path[len(glob_str):]
             except OSError:
                 continue
-        else:
-            return ''
-        return wwn_path[len(glob_str):]
+        return ''
 
     def get_sysfs_wwid(self, device_names):
         """Return the wwid from sysfs in any of devices in udev format."""
@@ -262,29 +240,23 @@ class LinuxSCSI(executor.Executor):
         """
         # No used path happens on failed attachs, when we don't care about
         # individual flushes.
-        if not path_used:
+        # When Nova used a multipath we don't need to do individual flushes.
+        if not path_used or was_multipath:
             return False
 
+        # We need to flush the single path that was used.
+        # For encrypted volumes the symlink has been replaced, so realpath
+        # won't return device under /dev but under /dev/disk/...
         path = os.path.realpath(path)
         path_used = os.path.realpath(path_used)
+        return path_used == path or '/dev' != os.path.split(path_used)[0]
 
-        # Need to flush this device if we used this specific path.  We check
-        # this before checking if it's multipath in case we don't detect it
-        # being multipath correctly (as in bug #1897787).
-        if path_used == path:
-            return True
-
-        # We flush individual path if Nova didn't use a multipath and we
-        # replaced the symlink to a real device with a link to the decrypted
-        # DM.  We know we replaced it because it doesn't link to /dev/XYZ,
-        # instead it maps to /dev/mapped/crypt-XYZ
-        return not was_multipath and '/dev' != os.path.split(path_used)[0]
-
-    def remove_connection(self, devices_names, force=False, exc=None,
-                          path_used=None, was_multipath=False):
+    def remove_connection(self, devices_names, is_multipath, force=False,
+                          exc=None, path_used=None, was_multipath=False):
         """Remove LUNs and multipath associated with devices names.
 
         :param devices_names: Iterable with real device names ('sda', 'sdb')
+        :param is_multipath: Whether this is a multipath connection or not
         :param force: Whether to forcefully disconnect even if flush fails.
         :param exc: ExceptionChainer where to add exceptions if forcing
         :param path_used: What path was used by Nova/Cinder for I/O
@@ -293,17 +265,19 @@ class LinuxSCSI(executor.Executor):
         """
         if not devices_names:
             return
+        multipath_name = None
         exc = exception.ExceptionChainer() if exc is None else exc
-
-        multipath_dm = self.find_sysfs_multipath_dm(devices_names)
         LOG.debug('Removing %(type)s devices %(devices)s',
-                  {'type': 'multipathed' if multipath_dm else 'single pathed',
+                  {'type': 'multipathed' if is_multipath else 'single pathed',
                    'devices': ', '.join(devices_names)})
-        multipath_name = multipath_dm and self.get_dm_name(multipath_dm)
-        if multipath_name:
-            with exc.context(force, 'Flushing %s failed', multipath_name):
-                self.flush_multipath_device(multipath_name)
-                multipath_name = None
+
+        if is_multipath:
+            multipath_dm = self.find_sysfs_multipath_dm(devices_names)
+            multipath_name = multipath_dm and self.get_dm_name(multipath_dm)
+            if multipath_name:
+                with exc.context(force, 'Flushing %s failed', multipath_name):
+                    self.flush_multipath_device(multipath_name)
+                    multipath_name = None
 
         for device_name in devices_names:
             dev_path = '/dev/' + device_name
